#include "display.h"

void display_initialize (void)
{
	// Reset display by holding RST low for a while.
	DISPLAY_RST = 0;
	delay_ms(50); // Minimal 1 us reset low time.
	DISPLAY_RST = 1;

	// Configuire display orientation.
	//display_write_command(LCD_SET_ADC_NOR);
	//display_write_command(LCD_SET_SHL_REV);

	display_write_command(LCD_SET_ADC_REV);
	display_write_command(LCD_SET_SHL_NOR);

	display_write_command(LCD_SET_BIAS_0);

	// Power up the volatge converters one by one.
	display_write_command(LCD_PWR_CTL | 1);
	delay_ms(100);
	display_write_command(LCD_PWR_CTL | 3);
	delay_ms(100);
	display_write_command(LCD_PWR_CTL | 7);
	
	display_write_command(LCD_REG_RESISTOR | 5);
	display_write_command(LCD_REF_VOLT_MODE);
	display_write_command(LCD_REF_VOLT_REG | 18);

	delay_ms(100);

	display_write_command(LCD_DISP_ON);
}

void display_write_command (unsigned char command)
{
	// Wait for display to become idle/available.
	while(display_read_status() & DISPLAY_STATUS_BUSY);

	DISPLAY_CS = 0;
	DISPLAY_RS = 0;
	DISPLAY_RW = 0;
	
	// Configure data bus as output.
	DISPLAY_DATA_DDR = DIRECTION_OUTPUT;

	DISPLAY_DATA = command;

	DISPLAY_E = 1;
	delay_ms(1); // minimal delay 120 ns
	DISPLAY_E = 0;

	DISPLAY_CS = 1;
}


void display_write_data (unsigned char data)
{
	// Wait for display to become idle/available.
	while(display_read_status() & DISPLAY_STATUS_BUSY);

	DISPLAY_CS = 0;
	DISPLAY_RS = 1;
	DISPLAY_RW = 0;
	
	// Configure data bus as output.
	DISPLAY_DATA_DDR = DIRECTION_OUTPUT;

	DISPLAY_DATA = data;

	DISPLAY_E = 1;
	delay_ms(1); // minimal delay 120 ns
	DISPLAY_E = 0;

	DISPLAY_CS = 1;
}

unsigned char display_read_status (void)
{
	volatile unsigned char temp;

	DISPLAY_CS = 0;
	DISPLAY_RS = 0;
	
	DISPLAY_RW = 1;

	// Switch to status read mode.
	DISPLAY_E = 1;

	// Configure data bus for input operations.
	trisd = 0xff;

	temp = DISPLAY_DATA;

	DISPLAY_E = 0;
	DISPLAY_CS = 1;

	return temp;
}

unsigned char display_read_data (void)
{
	volatile unsigned char temp;

	DISPLAY_CS = 0;
	DISPLAY_RS = 1;
	
	DISPLAY_RW = 1;

	// Switch to status read mode.
	DISPLAY_E = 1;
	delay_us(1);
	DISPLAY_E = 0;
	delay_us(1);
	DISPLAY_E = 1;
	delay_us(1);

	// Configure data bus for input operations.
	DISPLAY_DATA_DDR = DIRECTION_INPUT;

	temp = DISPLAY_DATA;

	DISPLAY_E = 0;
	DISPLAY_CS = 1;

	return temp;
}

void display_clear (void)
{
	unsigned char column, page;

	for (page = 0; page < DISPLAY_PAGE_COUNT; ++page) {
		display_goto(0, page);
		for (column = 0; column < DISPLAY_COLUMN_COUNT; ++column) {
			display_write_data(0x00);
		}
	}
	display_goto(0, 0);
}

void display_bitmap (const unsigned char *data0, const unsigned char *data1,
	const unsigned char *data2, const unsigned char *data3)
{
	unsigned char column, page;

	for (page = 0; page < 2; ++page) {
		display_goto(0, page);
		for (column = 0; column < DISPLAY_COLUMN_COUNT; ++column) {
			display_write_data(*(data0 + page * DISPLAY_COLUMN_COUNT + column));
		}
	}

	for (page = 0; page < 2; ++page) {
		display_goto(0, page + 2);
		for (column = 0; column < DISPLAY_COLUMN_COUNT; ++column) {
			display_write_data(*(data1 + page * DISPLAY_COLUMN_COUNT + column));
		}
	}

	for (page = 0; page < 2; ++page) {
		display_goto(0, page + 4);
		for (column = 0; column < DISPLAY_COLUMN_COUNT; ++column) {
			display_write_data(*(data2 + page * DISPLAY_COLUMN_COUNT + column));
		}
	}

	for (page = 0; page < 2; ++page) {
		display_goto(0, page + 6);
		for (column = 0; column < DISPLAY_COLUMN_COUNT; ++column) {
			display_write_data(*(data3 + page * DISPLAY_COLUMN_COUNT + column));
		}
	}

	display_goto(0, 0);
}

void display_goto (unsigned char column, unsigned char page)
{
	display_write_command(LCD_SET_PAGE | page);

	display_write_command(LCD_SET_COL_HI | ((column + DISPLAY_COLUMN_OFFSET) >> 4));
	display_write_command(LCD_SET_COL_LO | ((column + DISPLAY_COLUMN_OFFSET) & 0x0f));
}

void display_set_pixel (unsigned char x, unsigned char y, unsigned char color)
{
	unsigned char page, remainder, pixel_value;

	if (x > DISPLAY_WIDTH || y > DISPLAY_HEIGHT) {
		return;
	}

	page = y / 8;
	remainder = y % 8;

	display_goto(x, page);
	pixel_value = display_read_data();

	if (color) {
		pixel_value |= (1 << remainder);
	} else {
		pixel_value &= ~(1 << remainder);
	}
	display_goto(x, page);
	display_write_data(pixel_value);
}

void display_draw_line (unsigned char x1, unsigned char y1,
	unsigned char x2, unsigned char y2)
{
	unsigned char i;
	char tmp, y;

	// Swap values if x2 is smaller than x1.
	if (x2 < x1) {
		i = x1; x1 = x2; x2 = i;
		i = y1; y1 = y2; y2 = i;
	}

	tmp = (y2 - y1) / (x2 - x1);

	y = y1;
	for (i = x1; i < x2; ++i) {
		display_set_pixel(i, y, 1);
		y += tmp;
	}

}

void display_draw_string (unsigned char x, unsigned char y,
	display_fonts_t font, const char *string)
{
	unsigned char *p;

	if (font == 0 || string == 0) { return; }

	for (p = string; *p != '\0'; ++p) {
		display_draw_character(x, y, font, *p);
	}
}

void display_draw_character (unsigned char x, unsigned char y,
	display_fonts_t font, char character)
{
	unsigned char i, column, row;
	unsigned char buffer[DISPLAY_FONT_LARGE_BYTESIZE];

	if (font == 0) { return; }

	switch (character)
	{
	case '0':
		for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_0[i];
		}
		break;
	case '1':
		for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_1[i];
		}
		break;
	case '2':
		for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_2[i];
		}
		break;
	case '3':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_3[i];
		}
		break;
	case '4':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_4[i];
		}
		break;
	case '5':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_5[i];
		}
		break;
	case '6':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_6[i];
		}
		break;
	case '7':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_7[i];
		}
		break;
	case '8':
			for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_8[i];
		}
		break;
	case '9':
		for (i = 0; i < DISPLAY_FONT_LARGE_BYTESIZE; ++i) {
			buffer[i] = font_large_9[i];
		}
		break;
	default:
		return;
	}

	for (column = 0; column < DISPLAY_FONT_LARGE_WIDTH; ++column) {
		for (row = 0; row < DISPLAY_FONT_LARGE_HEIGHT; ++row) {
			//display_set_pixel(x + column, y + row, *(glyph + (column * DISPLAY_FONT_LARGE_WIDTH) + row));
			display_set_pixel(x + column, y + row, buffer[((column * DISPLAY_FONT_LARGE_WIDTH) + row)]);
		}
	}
	
}